/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: ansiio.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "memory.h"
#include "ansiconvert.h"

static unsigned char *buffer = NULL;
static unsigned long size = 0;

void FreeANSIReadWriteBuffer()
{
	myfree(buffer);
	buffer = NULL;
	size = 0;
}

int WriteANSIDouble(FILE *f, double *a, int n)
{
	int i;
	unsigned int sz = n * ANSI_DOUBLE_SIZE;
	if(sz > size) 
	  buffer = myrealloc(buffer, size = sz);
	for(i=0; i<n; ++i)
	  doubleansi(buffer + i * ANSI_DOUBLE_SIZE, a[i]);

	if(fwrite(buffer, sz, 1, f) != 1)
	  return 1;

	return 0;
}

int WriteANSIInt(FILE *f, int *a, int n)
{
	int i;
	unsigned int sz = n * ANSI_INT_SIZE;
	if(sz > size) 
	  buffer = myrealloc(buffer, size = sz);
	for(i=0; i<n; ++i)
	  intansi(buffer + i * ANSI_INT_SIZE, a[i]);

	if(fwrite(buffer, sz, 1, f) != 1)
	  return 1;

	return 0;
}

int ReadANSIDouble(FILE *f, double *a, int n)
{
	int i;
	unsigned int sz = n * ANSI_DOUBLE_SIZE;
	if(sz > size) 
	  buffer = myrealloc(buffer, size = sz);

	if(fread(buffer, sz, 1, f) != 1)
	  return 1;

	for(i=0; i<n; ++i)
	  a[i] = ansidouble(buffer + i * ANSI_DOUBLE_SIZE);

	return 0;
}

int ReadANSIInt(FILE *f, int *a, int n)
{
	int i;
	unsigned int sz = n * ANSI_INT_SIZE;
	if(sz > size) 
	  buffer = myrealloc(buffer, size = sz);

	if(fread(buffer, sz, 1, f) != 1)
	  return 1;

	for(i=0; i<n; ++i)
	  a[i] = ansiint(buffer + i * ANSI_INT_SIZE);

	return 0;
}

