/* A small set of routines to aid in timing. 
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: dmdtimer.c,v 1.5 1995/12/19 18:08:33 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "memory.h"
#include "dmdtimer.h"

#define   RUSAGE_SELF     0
#define   RUSAGE_CHILDREN     -1

#define GETSEC(val) (val.tv_sec + 1e-6 * val.tv_usec)

#ifndef MyClock /* this should be a macro if possible! */

/* Return the elapsed CPU time in seconds, counting ourselves and
 * all child processes.  On a multi-node machine,
 */
double MyClock(void)
{
#if defined(SUNMOS) || defined(ncube)
	/* With this interface, we have to settle for returning
	 * zero the first time this is called.
	 */
	static long zerosec = 0;
	static int boot = 0;
	long sec;
	int usec;
	sec = n_time(&usec);
	if(!boot)
	{
	  zerosec = sec;
	  boot = 1;
	}
	sec -= zerosec;
	return ((double) sec) + 1.e-6 * ((double) usec);
#elif defined(_CRAY)
	return (clock() / CLOCKS_PER_SEC);
#else
	struct rusage ru;
	double result = 0.0;
	getrusage(RUSAGE_SELF, &ru);
	result += GETSEC(ru.ru_utime);
	getrusage(RUSAGE_CHILDREN, &ru);
	result += GETSEC(ru.ru_utime);
	return result;
#endif
}

#endif /* MyClock */

double WallClock(void)
{
#if defined(SUNMOS)
	return MyClock();
#else
	time_t t = time(NULL);
	static time_t t_zero;
	static int init = 1;
	if(init) {
	  t_zero = t;
	  init = 0;
	}
	return difftime(t, t_zero);
#endif
}

/* Get standardized system resources.
 */
void GetResources(ResourceInfo *ri)
{
#if defined(SUNMOS) || defined(ncube) || defined(_CRAY)
	ri->tuser = MyClock();
	ri->tsystem = 0.0; /* XXX */
	ri->faults = 0; /* no paging -- no faults possible */
#else
	struct rusage ru;
	getrusage(RUSAGE_SELF, &ru);

	ri->tuser = GETSEC(ru.ru_utime);
	ri->tsystem = GETSEC(ru.ru_stime);
	ri->faults = ru.ru_majflt;
#endif
}

