/* This file is part of David Deaven's personal library.
 * Copyright (c) 1995 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: error.c,v 1.2 1996/05/29 21:11:16 deaven Exp $
 */

/* libdmd error handling
 */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include "tools.h"

int dmd_errno = DMD_NORMAL;	/* errno */

static int sys_errno = 0;		/* saved errno from system */
static char *dmd_sys_message;

static char *dmd_error_names[DMD_MAXERROR] = {
	"normal operation",
	"system error",
	"invalid argument",
	"auxillary software failed"
	"resource unavailable",
	"invalid input"
};

/* print the error message associated with dmd_errno to stderr,
 * and clear the error condition.
 */
int dmd_perror(char *s)
{
	int r = dmd_errno;
	if(dmd_errno != DMD_NORMAL) 
	{
	  fprintf(stderr, "%s: %s", s, dmd_error_names[dmd_errno]);
	  if(dmd_errno == DMD_SYSTEM) 
	    fprintf(stderr, ": %s: %s\n",
	      dmd_sys_message, strerror(sys_errno));
	  else
	    fprintf(stderr, "\n");
	  
	  dmd_errno = DMD_NORMAL;
	}
	return r;
}

/* transfer the system errno to the temporary sys_errno
 * and return a -1.
 */
int dmd_system_error(char *s)
{
	sys_errno = errno;
	dmd_sys_message = s;
	dmd_errno = DMD_SYSTEM;
	return -1;
}

void dmd_fatal_error(char *s)
{
	dmd_perror(s);
	exit(1);
}
