/* Read FORTRAN records.
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: f77read.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <sys/types.h>
#include "tools.h"
#ifdef NOIEEE
#include "ansiconvert.h"
#endif /* NOIEEE */

/* Read one FORTRAN record -- analog of fread()
 *
 * David Deaven 23 February 1994 deaven@ishmael.ameslab.gov 
 */
int f77read(void *ptr, int size, int nobj, FILE *stream)
{
  int start, stop, n;
  if((n = fread(&start, sizeof(int), 1, stream))!=1) {
  	fprintf(stderr, "f77read(): cannot read start word (read %d)\n", n);
  	return 0;
  }
  if((n = fread(ptr, (size_t) size, (size_t) nobj, stream))!=nobj) {
  	fprintf(stderr, "f77read(): cannot read all data (read %d/%d)\n", n, nobj);
  	return 0;
  }
  if((n = fread(&stop, sizeof(int), 1, stream))!=1) {
  	fprintf(stderr, "f77read(): cannot read stop word (read %d)\n", n);
  	return 0;
  }
#ifdef NOIEEE
  if(size == sizeof(double)) { /* assume it's a double */
    int i;
    double d;
    for(i=0; i<nobj; ++i) {
      d=ansidouble((unsigned char *) ptr+i*size);
      ((double *)ptr)[i] = d;
    }
  } else if(size == sizeof(int)) { /* assume it's an int */
    int i, d;
    for(i=0; i<nobj; ++i) {
      d=ansiint((unsigned char *) ptr+i*size);
      ((int *)ptr)[i] = d;
    }
  } else fprintf(stderr,
    "f77read: warning: size %d f77 object may not convert\n", size);
#endif /* NOIEEE */
  return nobj;
}

#ifdef NOIEEE
static int told=0;
#endif
int f77write(void *ptr, int size, int nobj, FILE *stream)
{
  int n, start=8, stop=8;
#ifdef NOIEEE
  if(!told) {
    fprintf(stderr,"Warning! this code writes non-IEEE data\n");
    told = 1;
  }
#endif /* NOIEEE */
  if((n = fwrite(&start, sizeof(int), 1, stream))!=1) {
  	fprintf(stderr, "f77write(): cannot write start word (wrote %d)\n", n);
  	return 0;
  }
  if((n = fwrite(ptr, (size_t) size, (size_t) nobj, stream))!=nobj) {
  	fprintf(stderr, "f77write(): cannot write all data (wrote %d/%d)\n", n, nobj);
  	return 0;
  }
  if((n = fwrite(&stop, sizeof(int), 1, stream))!=1) {
  	fprintf(stderr, "f77write(): cannot write stop word (wrote %d)\n", n);
  	return 0;
  }
  return nobj;
}
