/* GENETIC ALGORITHM AND RELATED CODE
 * Copyright (c) 1995 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: file.c,v 1.5 1996/05/29 21:11:16 deaven Exp $
 * $Author: deaven $
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tools.h"

FILE *fopen_with_default(char *fn, char *mode, FILE *fdefault)
{
	FILE *f = NULL;

	if(fn != NULL) {
	  if(strcmp(fn, "-") == 0) {
	    switch(mode[0]) {
	      case 'a':
	      case 'w':
		f = stdout;
		break;
	      case 'r':
		f = stdin;
		break;
	      default:
		f = NULL;
		break;
	    }
	  } else {
	    f = fopen(fn, mode);
	  }
	}

	if(f == NULL) {
	  if((f = fdefault) == NULL) {
	    static char buf[FILENAME_MAX + 64];
	    sprintf(buf, "cannot open `%s' for \"%s\"\n", fn, mode);
	    dmd_errno = DMD_RESOURCE_UNAVAILABLE;
	    dmd_fatal_error(buf);
	  }
	}

	return f;
}

/* Given a colon-separated path list and a filename, return a stream
 * for the first existing file, or NULL if none are found.
 */
FILE *fopen_path(char *pathlist, char *filename, char *mode)
{
  unsigned int i;
  FILE *try;
  char buf[FILENAME_MAX];

  while(*pathlist != '\0')
  {
    for(i=0; pathlist[i]!=':' && pathlist[i]!='\0'; ++i) {
      if(i > FILENAME_MAX) {
	fprintf(stderr, "fopenpath(): bad path spec `%s'\n", pathlist);
	return NULL;
      }
    }

    strncpy(buf, pathlist, i);
    buf[i] = '\0';
    strcat(buf, "/");
    strcat(buf, filename);

    if((try = fopen(buf, mode)) != NULL) return try;

    pathlist +=i;

    if(*pathlist == ':') ++pathlist;
  }
  return NULL;
}

