/* This file is part of David Deaven's personal library
 * Copyright (c) 1995 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: findfile.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "tools.h"

/* Search for the complete path specification of the file whose
 * name is given.  The search proceeds in all directories in the
 * search path, which may be given directly, or may be looked up
 * from an environment variable if the first character in "searchpath"
 * is "@".  If "pathspec" is NULL, only the current working directory
 * is searched.
 *
 * If the file has an absolute starting "/", no search is done.
 */
char *findfile(char *name, char *pathspec)
{
	int i;
	char c, *path = ".";
	struct stat stbuf;
	static char file[FILENAME_MAX];

	if(*name == '/') {
	  if(stat(name, &stbuf)==0) return name;
	  return NULL;
	}

	if(pathspec != NULL && *pathspec == '@') {
	  if((path = getenv(pathspec + 1)) == NULL) path = ".";
	}
	else
	  path = pathspec;

	while(*path != '\0')
	{
	  i = 0;
	  while((c = *(path++))!=':' && c!='\0')
	    if(i<FILENAME_MAX-2) file[i++] = c;
	  file[i] = '\0';
	  (void) strcat(file, "/");
	  (void) strncat(file, name, FILENAME_MAX-strlen(file));
	  if(stat(file, &stbuf)==0) return file;
	  if(c == '\0') break;
	}

	return NULL;
}

