/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: fit_ds.c,v 1.2 1996/05/29 21:11:16 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "memory.h"
#include "tools.h"
#include "random.h"
#include "fit.h"


struct resources {
	Fit fit;
	int np;			/* number of varied parameters */
	int *which;		/* index to varied parameters */
	double ftol;		/* fractional convergence */
	int iter;		/* maximum iterations */
};

static int ds_fun(double *x, double *e, void *res)
{
	struct resources *r = res;
	int i;

	for(i=0; i<r->np; ++i)
	  r->fit->par[r->which[i]] = x[i];
	r->fit->changed = 1;

	i = FitEvaluate(r->fit);
	*e = r->fit->rms;

        return i;
}

int FitByDownhillSimplex(Fit fit, char *options, FILE *fout)
{
	int i, j, r = 0, nfunk;
	double **p, *e, dp;

	struct resources res;
	res.fit = fit;
	res.which = imalloc(fit->npar);
	res.ftol = 1.e-8;
	res.iter = 128;

	for(i=res.np=0; i<fit->npar; ++i)
	  if(fit->fitpar[i]) res.which[res.np++] = i;
	if(res.np < 1) {myfree(res.which); return 0;}

	/* Things required by ds_fun()
	 */
	e = dmalloc(res.np + 1);
	p = dpmalloc(res.np + 1);
	for(i=0; i<(res.np + 1); ++i)
	{
	  p[i] = dmalloc(res.np);
	  for(j=0; j<res.np; ++j) {
	    dp = FitTrialRange(fit, res.which[j]);
	    p[i][j] = fit->par[res.which[j]] + dp;
	  }
	  ds_fun(p[i], e + i, &res);
	}

	{
	  int this_iter, this_nfunk;
	  nfunk = 0;
	  while(nfunk < res.iter)
	  {
	    this_iter = 25; 
	    if(this_iter > (res.iter - nfunk)) this_iter = res.iter - nfunk;
	    r = amoeba(p, e, res.np, res.ftol, this_iter, ds_fun, &res,
	      &this_nfunk);
	    if(r == -1) r = 0; /* ignore `too many iterations' */
	    nfunk += this_nfunk;
	    if(r || this_nfunk < this_iter) break;
	  }
	  if(fout)
	    fprintf(fout, "FitByDownhillSimplex:"
	      " amoeba made %d moves (%d allowed)\n", nfunk, res.iter);
	  if(r && fout)
	    fprintf(fout, "FitByDownhillSimplex:"
	      "amoeba terminated with error %d\n", r);
	}

	ds_fun(p[0], e, &res); /* set to best point */

	for(i=0; i<(res.np + 1); ++i) myfree(p[i]);
	myfree(p);
	myfree(e);
	myfree(res.which);

	return r;
}

