/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: fprintnl.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <stdlib.h>
#include "geometry.h"
#include "neighbor.h"
#include "memory.h"

struct neib {
	int n;
	double r;
};

static int cmp(const void *a, const void *b)
{
	if(((struct neib *) a)->r > ((struct neib *) b)->r) return 1;
	return -1;
}

static void pout(FILE *f, Geometry *g, struct nl_s *nl, int i, int nmax,
	struct neib *list)
{
      int j, jnl, nn[3];
      double rij, dxij[3];
      unsigned int n;
      for(n=0, jnl=nl->mrkr[i]; jnl<(nl->mrkr[i+1]); ++jnl, ++n)
      {
	getnl(g, i, nl->list[jnl], &j, &rij, dxij, nn);
	list[n].n = j+1;
	list[n].r = rij;
      }
      qsort(list, n, sizeof(struct neib), cmp);
      fprintf(f, "%d (%d): ", i+1, (int) n);
      for(j=0; j<n && j<nmax; ++j)
      {
	if((j+1) % 6 == 0) fprintf(f, "\n. ");
	fprintf(f, "%d(%.3f) ", list[j].n, list[j].r);
      }
      fprintf(f, "\n");
}

void fprintnlone(FILE *f, Geometry *g, struct nl_s *nl, int i, int nmax)
{
    struct neib *list = mymalloc(nl->size * sizeof(struct neib));
    fprintf(f, "%4s    \t%s\n", "atom", "neighbor(ascending distance)");
    pout(f, g, nl, i, nmax, list);
    myfree(list);
}

/* Print a neighbor list in readable form, tabulating at most
 * nmax neighbors per atom.
 */
void fprintnl(FILE *f, Geometry *g, struct nl_s *nl, int nmax)
{
    int i;
    struct neib *list = mymalloc(nl->size * sizeof(struct neib));
    fprintf(f, "geometry: \"%s\"\nneighbor list cutoff: %.3f bohr\n", 
	g->name, nl->rc);
    fprintf(f, "%4s     \t%s\n", "atom", "neighbor(ascending distance)");
    for(i=0; i<(g->n); ++i) pout(f, g, nl, i, nmax, list);
    myfree(list);
}
