/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gboxcenter.c,v 1.3 1996/07/09 16:36:10 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "geometry.h"
#include "tools.h"

/* Translate to box "center of number" and remove extra periodic
 * displacements.
 */
Geometry *GboxCenter(Geometry *g)
{
  int i, m;
  double x0[3];

  if(!(g->period & GP_INCOMPLETE)) {
    GgetBoxCenter(g, x0);
    for(m=0; m<3; m++) {
      for(i=0; i<(g->n); i++) {
	g->x[m+3*i] -= x0[m];
      }
    }
  }

  return g;
}

double *GgetBoxCenter(Geometry *g, double center[3])
{
  int i, m, mp;

  center[0] = center[1] = center[2] = 0.0;

  for(m=0; m<3; m++) {
    mp = g->period & GP_PDIR(m) ? 1 : 0;
    for(i=0; i<(g->n); i++) {
      if(mp) g->x[m+3*i] -= anint(g->x[m+3*i]);
      center[m] += g->x[m+3*i];
    }
  }
  for(m=0; m<3; m++) center[m] /= (double) g->n;

  return center;
}
