/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "geometry.h"
#include "memory.h"

/* Concatenate two geometries. 
 * The box transformation and periodicity of the first
 * geometry is used for the result.
 *
 * Flags are present iff the first geometry has them.  If it 
 * does and the second does not, zeros are inserted.
 */
Geometry *Gcat(Geometry *dst, Geometry *g1, Geometry *g2)
{
  int i, j;
  double x[3];

  if(dst == NULL)
    dst = Gmalloc(g1->n + g2->n, g1->name);
  else
    dst = Grealloc(dst, g1->n + g2->n);

  if(g1->flags != NULL)
    dst->flags = myrealloc(dst->flags, (g1->n + g2->n) * sizeof(int));
  else
  {
    myfree(dst->flags);
    dst->flags = NULL;
  }
  dst->period = g1->period;
  for(i=0; i<9; ++i)
  {
    dst->box[i] = g1->box[i];
    dst->boxi[i] = g1->boxi[i];
  }
  for(i=0; i<(g1->n); ++i)
  {
    for(j=0; j<3; ++j)(dst->x+3*i)[j] = (g1->x+3*i)[j];
    myfree(dst->aname[i]); dst->aname[i] = mystrdup(g1->aname[i]);
    if(g1->flags != NULL) dst->flags[i] = g1->flags[i];
  }
  for(i=0; i<(g2->n); ++i)
  {
    (void) mmul3(g2->box, g2->x+3*i, x);
    (void) mmul3(g1->boxi, x, dst->x+3*(i + g1->n));
    myfree(dst->aname[i+g1->n]); dst->aname[i+g1->n] = mystrdup(g2->aname[i]);
    if(g1->flags != NULL)
      dst->flags[i + g1->n] = (g2->flags != NULL ? g2->flags[i] : 0);
  }
  i = (g1->name == NULL ? 0 : strlen(g1->name)) +
      (g2->name == NULL ? 0 : strlen(g2->name));
  if(i>0) {
    if(dst->name != NULL)myfree(dst->name);
    dst->name = mymalloc((i+5) * sizeof(char));
    strcpy(dst->name, g1->name);
    strcat(dst->name, " + ");
    strcat(dst->name, g2->name);
  }
  return dst;
}
