/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gdist.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include "tools.h"
#include "geometry.h"

/* Determine the distance between two atoms and the 
 * box space displacement vector x(j) - x(i).
 */
double Gdisp2(Geometry *g, int i, int j, double x[3])
{
	int m;
	double dx, r, *b = g->box;
	for(m=0; m<3; ++m) {
	  x[m] = g->x[j*3+m] - g->x[i*3+m];
	  if((g->period >> m) & 1) x[m] -= anint(x[m]);
	}
	r = 0.0;
	for(m=0; m<3; ++m)
	{
	  dx = -(x[0]*b[m*3]+x[1]*b[m*3+1]+x[2]*b[m*3+2]);
	  r += dx*dx;
	}
	return r;
}

double Gdisp(Geometry *g, int i, int j, double x[3])
{
	return sqrt(Gdisp2(g, i, j, x));
}

/* Determine the distance between two atoms.
 */
double Gdist(Geometry *g, int i, int j)
{
	double dx[3];
	return Gdisp(g, i, j, dx);
}

