/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: getargv.c,v 1.4 1996/02/23 00:42:31 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <ctype.h>
#include "memory.h"
#include "tools.h"

/* Create a new argument vector, composed of the supplied 
 * arguments from 0 to (where - 1), followed by arguments
 * parsed from s, followed by arguments where to argc - 1 
 * from the supplied list.
 *
 * The new vector is allocated here, as are all of the argument
 * strings, regardless of their origin.
 */
char **getargv(char *s, int where, int *argc, char **argv)
{
	int i, n = 0;
	char c, *save, **new = NULL;

	if(where < 0) where = 0;
	if(where > (*argc)) where = (*argc);

	if(where > 0) {
	  new = myrealloc(new, (n + where) * sizeof(*new));
	  for(i=0; i<where; ++i) new[n++] = mystrdup(argv[i]);
	}

	s = save = mystrdup(s); /* writable copy */
	while((*s) != '\0')
	{
	  while(isspace(*s)) ++s;
	  if(!isprint(*s)) break;
	  new = myrealloc(new, (n + 1) * sizeof(*new));
	  new[n] = s; /* placeholder */
	  while(isprint(*s) && !isspace(*s)) ++s;
	  c = *s; *s = '\0'; new[n] = mystrdup(new[n]); *s = c;
	  n++;
	}
	myfree(save);

	if(where < (*argc)) {
	  new = myrealloc(new, (n + (*argc) - where) * sizeof(*new));          
	  for(i=where; i<(*argc); ++i) new[n++] = mystrdup(argv[i]);
	}

	*argc = n;

	new = myrealloc(new, (n + 1) * sizeof(*new));
	new[n++] = NULL;

	return new;
}

/* Insert arguments parsed from the file into the argument 
 * vector at the specified position. The original argument
 * vector is unchanged.
 */
char **fgetargv(FILE *f, int where, int *argc, char **argv)
{
	int i;
	char buf[128], *s, **save = NULL;

	if(where < 0) where = 0;
	if(where > (*argc)) where = (*argc);

	while(fgets(buf, 128, f) != NULL) {
	  for(s=buf; isspace(*s); ++s);
	  if(*s != '#') {
	    i = *argc;
	    argv = getargv(buf, where, argc, argv);
	    where += (*argc) - i;
	    if(save) {
	      for(i=0; save[i]; ++i) myfree(save[i]);
	      myfree(save);
	    }
	    save = argv; /* all except the first time */
	  }
	}

	return argv;
}
