/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gmopac.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "geometry.h"
#include "memory.h"

#define ANGSTROM 0.529177

static int blankline(char *line)
{
	while(*line != '\0') {
	  if(!isspace(*line)) return 0;
	  ++line;
	}
	return 1;
}

/* Read the next cartesian coordinates in the MOPAC
 * output file on the stream f.
 */
Geometry *GreadMPCOutput(FILE *f)
{
	char buf[128];
	int i, n = 0;
	double x[3];
	char *atom;
	Geometry *g = NULL;

	while(fgets(buf, 128, f) &&
	      strstr(buf, "CARTESIAN COORDINATES") == NULL);
	if(feof(f)) return NULL;
	for(i=0; i<3; ++i) fgets(buf, 128, f);
	if(feof(f)) return NULL;
	while(!feof(f) && fgets(buf, 128, f) && !blankline(buf)) {
	  atom = buf + 14;
	  for(i=0; atom[i] != ' '; ++i);
	  atom[i] = '\0';

	  if(sscanf(buf + 19, "%lf %lf %lf", x, x + 1, x + 2) != 3)
	    break;
	  for(i=0; i<3; ++i) x[i] /= ANGSTROM;
	  g = Grealloc(g, n + 1);
	  mmul3(g->boxi, x, g->x + 3*n);
	  g->aname[n] = mystrdup(atom);
	  ++n;
	}

	if(n > 0) {
	  g->name = mystrdup("Created from MOPAC output");
	  return g;
	}

	return NULL;
}

/* Print MOPAC geometry specification (but no other part of the
 * MOPAC input file) on file f.
 * Specify that all coordinates be optimized.
 */
void GprintMPC(FILE *f, Geometry *g)
{
	int i;
	double x[3];

	for(i=0; i<g->n; ++i) {
	  mmul3(g->box, g->x + 3*i, x);
	  fprintf(f, "%s   %16.8g 1 %16.8g 1 %16.8g 1\n", g->aname[i],
	    x[0] * ANGSTROM, x[1] * ANGSTROM, x[2] * ANGSTROM);
	}
}
