/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gprint.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "geometry.h"
#include "memory.h"

#define ANGSTROM 0.529177  /* conversion from internal (atomic) */

/* Print a geometry description to the given stream.
 * Use xmol's "xyz" format (cannot preserve periodic boundarys,
 * or save flags).
 */
void GprintXYZ(FILE *f, Geometry *g)
{
  int i, j;
  double x[3];
  fprintf(f, "%d\n", g->n);
  fprintf(f, "%s\n", g->name);
  for(i=0; i<(g->n); ++i) {
    fprintf(f, "%s", g->aname[i] == NULL ? "X" : g->aname[i]);
    (void) mmul3(g->box, g->x+i*3, x);
    for(j=0; j<3; ++j)fprintf(f, " %.11f", x[j]*ANGSTROM);
    fprintf(f, "\n");
  }
}

/* Print a geometry, generally, to a file
 * Use internal (atomic) physical coordinates
 */
void Gprint(FILE *f, Geometry *g)
{
  int i;
  double x[3];
  fprintf(f, "general%s%s\n%s\n%d (%hd,%hd,%hd) 1.0\n",
    (g->flags != NULL ? " flags" : ""),
    (g->period & GP_INCOMPLETE ? " incomplete" : ""),
    g->name, g->n,
    g->period & 1, (g->period >> 1) & 1, (g->period >> 2) & 1);
  for(i=0; i<9; ++i)
    fprintf(f, "%s%.6f", (i%3 ? " " : "\n"), g->box[i]);
  fprintf(f, "\n\n");
  for(i=0; i<(g->n); ++i) {
    mmul3(g->box, g->x+3*i, x);
    fprintf(f, "%s %.11f %.11f %.11f",
      g->aname[i] == NULL ? "X" : g->aname[i], x[0], x[1], x[2]);
    if(g->flags != NULL)
      fprintf(f, " 0x%x\n", g->flags[i]);
    else 
      fprintf(f, "\n");
  }
}

/* Print a geometry, generally, to a file.
 * Use box coordinates.
 */
void Gprintbox(FILE *f, Geometry *g)
{
  int i;
  double *x;
  fprintf(f, "general lattice%s%s\n%s\n%d (%hd,%hd,%hd) 1.0\n",
    (g->flags != NULL ? " flags" : ""),
    (g->period & GP_INCOMPLETE ? " incomplete" : ""),
    g->name, g->n,
    g->period & 1, (g->period >> 1) & 1, (g->period >> 2) & 1);
  for(i=0; i<9; ++i)
    fprintf(f, "%s%.6f", (i%3 ? " " : "\n"), g->box[i]);
  fprintf(f, "\n\n");
  for(i=0; i<(g->n); ++i) {
    x = g->x+3*i;
    fprintf(f, "%s %.11f %.11f %.11f",
      g->aname[i] == NULL ? "X" : g->aname[i], x[0], x[1], x[2]);
    if(g->flags != NULL)
      fprintf(f, " 0x%x\n", g->flags[i]);
    else 
      fprintf(f, "\n");
  }
  fprintf(f, "\n");
}

