/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gseparate.c,v 1.4 1996/07/09 20:08:40 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "geometry.h"
#include "neighbor.h"
#include "tools.h"
#include "random.h"

#define MAX_ITERATIONS	128	/* maximum iterations */

/* Helper function to keep atoms in box.
 */
static void fixatom(Geometry *g, int i)
{
	int m;
	for(m=0; m<3; ++m)
	  if((g->period >> m) & 1)
	    (g->x + 3*i)[m] -= anint((g->x + 3*i)[m]);
}

/* Separate atoms that are too close together.  This cannot always
 * be done if the geometry is periodic for example. XXX
 */
Geometry *Gseparate(Geometry *g, double dmin)
{
  int i, j, n, m;
  int iter = MAX_ITERATIONS;
  double dx[3], d, rb, eps = dmin * 0.0001;
  do {
    n = 0;
    for(i=0; i<(g->n - 1); ++i)
      for(j=i+1; j<(g->n); ++j)
	if((d = Gdisp(g, i, j, dx)) < dmin)
	{
	  /* We move each atom this much away from the offending
	   * neighbor, in fraction of interatomic box space vector.
	   */
	  rb = 0.5 * (2.0 * eps + dmin - d) / (d + eps);
	  for(m=0; m<3; ++m)
	  {
	    (g->x+3*i)[m] -= dx[m] * rb;
	    (g->x+3*j)[m] += dx[m] * rb;
	  }
	  if(g->period & GP_PERIODIC)
	  {
	    fixatom(g, i);
	    fixatom(g, j);
	  }
	  ++n;
	}
  } while(iter-- && n > 0);

  if(n > 0) fprintf(stderr, "Gseparate: missed %d\n", n);

  return g;
}

/* OLD CODE */
#ifdef NEVER
Geometry *Gseparate(Geometry *g, double dmin)
{
  int i, j, n, m, jnl, nn[3];
  int iter = MAX_ITERATIONS;
  double d, dx[3], eps=dmin*0.0001;
  double f1, f2;
  struct nl_s *nl;

  nl = nlalloc(dmin);

  do {
    GCentralCell(g);
    newnl(g, nl);
    n = 0;
    for(i=0; i<(g->n); ++i)
      for(jnl=nl->mrkr[i]; jnl<nl->mrkr[i+1]; ++jnl)
      {
	getnl(g, i, nl->list[jnl], &j, &d, dx, nn);
	(void) mtrn3(g->box, g->x+3*i);
	(void) mtrn3(g->box, g->x+3*j);
	f1 = ran3();
	f2 = 1.0 - f1;
	for(m=0; m<3; ++m)
	{
	  (g->x+3*i)[m] -= dx[m] * f1 * (eps + dmin - d);
	  (g->x+3*j)[m] += dx[m] * f2 * (eps + dmin - d);
	}
	(void) mtrn3(g->boxi, g->x+3*i);
	(void) mtrn3(g->boxi, g->x+3*j);
	if(g->period & GP_PERIODIC)
	{
	  fixatom(g, i);
	  fixatom(g, j);
	}
	++n;
      }
  } while(iter-- > 0 && n > 0);

  if(n > 0) fprintf(stderr, "Gseparate: missed %d\n", n);

  nlfree(nl);
  return g;
#endif
