/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: histogram.h,v 1.2 1995/08/29 22:45:15 deaven Exp deaven $
 * $Author: deaven $
 */

#ifndef _HISTOGRAM_H_
#define _HISTOGRAM_H_

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	int m;		/* bin center in dx units */
	double z;	/* bin weight */
} Bindata;

typedef struct {
	int n;		/* number of allocated bins */
	double base;	/* base for bin x units (m=0) */
	double dx;	/* bin width */
	Bindata *data;	/* array of bins */
} Histogram;

Histogram *Histmalloc(double dx);
Histogram *Histclear(Histogram *h);
void Histfree(Histogram *h);
int Histadd(Histogram *h, double x, double z);
void Histsort(Histogram *h);
void HistXYoff(FILE *f, Histogram *h, double xoff, double yoff);
void HistXY(FILE *f, Histogram *h);

void Histcollapse(Histogram *h);

#ifdef __cplusplus
}
#endif

#endif /* _HISTOGRAM_H_ */
