/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: memory.h,v 1.6 1996/05/16 22:09:13 deaven Exp deaven $
 * $Author: deaven $
 */

#ifndef _MEMORY_H_
#define _MEMORY_H_
#include <stdio.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

/* prototypes */

void *mymalloc(size_t size);
void *myrealloc(void *ptr, size_t size);
void myfree(void *ptr);
char *mystrdup(char *s);
void debug_mem(void);

void *mymalloc_x(size_t size, char *file, int line);
void *myrealloc_x(void *ptr, size_t size, char *file, int line);
void myfree_x(void *ptr, char *file, int line);
char *mystrdup_x(char *s, char *file, int line);
void mymemreport_x(FILE *f);

/* Special conditions for tracking memory */

#ifdef TRACKMEM

#define mymalloc(foo) mymalloc_x(foo, __FILE__, __LINE__)
#define myrealloc(foo, bar) myrealloc_x(foo, bar, __FILE__, __LINE__)
#define myfree(foo) myfree_x(foo, __FILE__, __LINE__)
#define mystrdup(foo) mystrdup_x(foo, __FILE__, __LINE__)
#define mymemreport(foo) mymemreport_x(foo)

/* Initially zero, this integer controls the heap allocation threshold
 * in bytes for reporting allocation changes.
 */
extern unsigned int memory_report_chunk;

#else

#define mymemreport(foo) ;

#endif /* TRACKMEM */

/* Macro definitions */

#define imalloc(n) ((int *) mymalloc((n) * sizeof(int)))
#define dmalloc(n) ((double *) mymalloc((n) * sizeof(double)))
#define dpmalloc(n) ((double **) mymalloc((n) * sizeof(double *)))

#if defined(__GNUC__)
#define NewPointerArray(P,N) \
  ((__typeof__(P)) mymalloc((N) * sizeof(*(P))))
#define ResizePointerArray(P,N) \
  ((__typeof__(P)) myrealloc(P, (N) * sizeof(*(P))))
#else
#define NewPointerArray(P,N) mymalloc((N) * sizeof(*(P)))
#define ResizePointerArray(P,N) myrealloc(P, (N) * sizeof(*(P)))
#endif

#define NewPointer(P) NewPointerArray(P, 1)

#ifdef __cplusplus
}
#endif

#endif /* _MEMORY_H_ */
