/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: nearnb.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include "neighbor.h"
#include "memory.h"
#include "tools.h"

#ifdef _I860_
#define sqrt(x) __builtin_sqrt(x)
#endif

/*==========================================================================
 * Find nearest neighbor of an atom.
 *========================================================================*/
int nearnb(Geometry *g, int i, double *d, double dx[3], int n[3])
{
  int j, rj=0, nn[3], m;
  double r, x[3], ddx[3], *b=g->box;

  *d = -1.0;
  for(j=0; j<(g->n); ++j) {
    if(i != j) {
      for(m=0; m<3; ++m) {
	x[m] = g->x[i*3+m] - g->x[j*3+m];
	if((g->period >> m) & 1) x[m] -= (nn[m]=anint(x[m]));
	else nn[m]=0;
      }
      for(m=0; m<3; ++m) ddx[m] = -(x[0]*b[m*3]+x[1]*b[m*3+1]+x[2]*b[m*3+2]);
      r = sqrt(ddx[0]*ddx[0] + ddx[1]*ddx[1] + ddx[2]*ddx[2]);
      if((*d) < 0.0 || r < (*d)) {
	*d = r;
	rj = j;
	n[0] = nn[0]; n[1] = nn[1]; n[2] = nn[2];
	dx[0] = ddx[0]/r; dx[1] = ddx[1]/r; dx[2] = ddx[2]/r;
      }
    }
  }
  return rj;
}
