/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: neighbor.h,v 1.4 1996/05/03 18:31:44 deaven Exp deaven $
 * $Author: deaven $
 */

#ifndef _NEIGHBOR_H_
#define _NEIGHBOR_H_
#include <stdio.h>
#include "dmd/geometry.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned long nl_t; /* holds 16 bit index plus three 5-bit cell offsets */

/* bit fields */
#define NLINDEXMASK 0x0000fffful
#define NLLXMASK (0x1f << 16)
#define NLLYMASK (0x1f << 21)
#define NLLZMASK (0x1f << 26)

/* 5 bit operations */
#define xnloff(n) ((n) < 0 ? (((-(n)) & 0x0f) | 0x10) : ((n) & 0x0f)) 
#define offnlx(n) ((n) & 0x10 ? (-((n) & 0x0f)) : ((n) & 0x0f))

/* make an nl_t: i = (0, 2^16-1) n[xyz] = (-15,15) */
#define asmnl(i,nx,ny,nz) \
  (((i) & NLINDEXMASK) | (xnloff(nx)<<16) | (xnloff(ny)<<21) | (xnloff(nz)<<26))

/* get parts of an nl_t */
#define getnlindex(list) ((list) & NLINDEXMASK)
#define getnlx(list) offnlx(((list) & NLLXMASK) >> 16)
#define getnly(list) offnlx(((list) & NLLYMASK) >> 21)
#define getnlz(list) offnlx(((list) & NLLZMASK) >> 26)

/* convenience macros to get high-level parts of an nl_t */
#define getnlnn(list,nn) \
  { \
    nn[0] = getnlx(list); \
    nn[1] = getnly(list); \
    nn[2] = getnlz(list); \
  }

#define getnldx(g,i,j,nn,dxb,dx) \
  { register int m; \
    for(m=0; m<3; ++m) (dxb)[m] = (g)->x[(i)*3+m] - (g)->x[(j)*3+m] - (nn)[m]; \
    for(m=0; m<3; ++m) (dx)[m] = -( \
      (dxb)[0]*(g)->box[m*3] + \
      (dxb)[1]*(g)->box[m*3+1] + \
      (dxb)[2]*(g)->box[m*3+2] ); \
  }

typedef struct nl_s NeighborList;
struct nl_s {
  double rc;		/* cutoff distance */
  int size;		/* number of pairs stored */
  int n;		/* number of atoms */
  int full;		/* include j < i AND j > i in list? */
  nl_t *list;		/* array of list entries */
  int *mrkr;		/* index array */
};

/* prototypes */

void nlfree(NeighborList *nl);
NeighborList *nlalloc(double rc);
void getnl(Geometry *g, int i, nl_t list, int *j,
	double *dij, double *dxij, int n[3]);
int nearnb(Geometry *g, int i, double *d, double dx[3], int n[3]);
void newnl(Geometry *g, NeighborList *nl);
void newnlm(Geometry *g, NeighborList *nl);

void fprintnl(FILE *f, Geometry *g, NeighborList *nl, int nmax);
void fprintnlone(FILE *f, Geometry *g, NeighborList *nl, int i, int nmax);

/* This will be removed in a later version */
int getcommonnl(int i, int j, NeighborList *nl, int *array);

int NLIntersect(int i, int j, NeighborList *nl, nl_t *array);

#ifdef __cplusplus
}
#endif

#endif /* _NEIGHBOR_H_ */
