/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: newnlm.c,v 1.3 1996/05/03 18:31:44 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include "neighbor.h"
#include "memory.h"

#ifdef _I860_
#define sqrt(x) __builtin_sqrt(x)
#endif

#define CHUNK 256 /* number of neighbor entries to allocate at a time */

/*==========================================================================
 * Generate a new neighbor list, with multiple connections between a 
 * central cell and the surrounding cells.
 * Some memory allocation is handled from within this routine.
 * Call with size==0 to begin.
 *
 * David Deaven 6 April 1994 deaven@ishmael.ameslab.gov 
 *========================================================================*/
void newnlm(Geometry *g, NeighborList *nl)
{
  int i, j, nn, m, n[3], nm[3];
  double r, dx[3], x[3], *b=g->box;

  if(nl->n == 0) {
    nl->mrkr = (int *)mymalloc((g->n+1) * sizeof(int));
    nl->n = g->n;
  }
  if(nl->size == 0) {
    nl->list = (nl_t *)mymalloc(CHUNK * sizeof(nl_t));
    nl->size = CHUNK;
  }
  if(g->n > nl->n) {
    nl->mrkr = (int *)myrealloc(nl->mrkr, (g->n + 1) * sizeof(int));
    nl->n = g->n;
  }
  for(i=nn=0; i<(g->n); ++i) {
    nl->mrkr[i]=nn;
    for(j=(nl->full ? 0 : i + 1); j<(g->n); ++j) {
      for(m=0; m<3; ++m) {
	x[m] = g->x[i*3+m] - g->x[j*3+m];
	if((g->period >> m) & 1) {
	  r = nl->rc / sqrt(b[m*3]*b[m*3] + b[m*3+1]*b[m*3+1] + b[m*3+2]*b[m*3+2]);
	  nm[m] = r + 1;
	} else nm[m] = 0;
      }
      for(n[0]=-nm[0]; n[0]<=nm[0]; ++n[0]) {
	for(n[1]=-nm[1]; n[1]<=nm[1]; ++n[1]) {
	  for(n[2]=-nm[2]; n[2]<=nm[2]; ++n[2]) {

	    if(i!=j || n[0]!=0 || n[1]!=0 || n[2]!=0) {
	
  for(m=0; m<3; ++m) dx[m] = 
    - ((x[0]-n[0])*b[m*3]+(x[1]-n[1])*b[m*3+1]+(x[2]-n[2])*b[m*3+2]);
  r = sqrt(dx[0]*dx[0] + dx[1]*dx[1] + dx[2]*dx[2]);
  if(r < nl->rc) {
    if(nn >= nl->size) {
      nl->list = (nl_t *) myrealloc(nl->list,
        (nl->size + CHUNK) * sizeof(nl_t));
      nl->size += CHUNK;
    }
    nl->list[nn++] = asmnl(j, n[0], n[1], n[2]);
  }

	    }
	  }
	}
      }
    }
  }
  nl->mrkr[g->n]=nn;
}

