/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: performance.c,v 1.1 1995/12/01 17:14:03 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "memory.h"
#include "dmdtimer.h"
#include "tools.h"

#define NMAX	(16 * 1024)	/* loop points */
#define NP	16		/* statistics points */

static double kflops(void)
{
	  int i, j, loop = 1;
	  double t0, t1;
	  double a, *m = dmalloc(NMAX);

	  do {
	    loop *= 4;
	    t0 = MyClock();
	    for(i=0; i<NMAX; ++i) {
	      a = i;
	      m[i] = a * a - 0.3456 * a; /* (3) */
	    }
	    for(j=0; j<loop; ++j) {
	      for(i=0; i<NMAX; ++i) {
		m[i] -= m[(i + 3 + NMAX / 2) % NMAX]; /* (1) */
		m[i] *= 0.1519 + 0.2536 * m[i]; /* (3) */
	      }
	    }
	    t1 = MyClock();
	  } while(t1 - t0 < 0.05);
	  a = 7.0 * NMAX * loop / (t1 - t0);

	  myfree(m);

	  return (a / 1024.0);
}

void performance(struct performance_s *perf)
{
	if(perf) {
	  int i;
	  double kflops_avg = 0.0;
	  for(i=0; i<NP; ++i)
	    kflops_avg += kflops();
	  perf->kflops = kflops_avg / NP;
	}
}
