/* Two-dimensional splines
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: spline6.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include <stdlib.h>
#include "spline.h"
#include "memory.h"

Spline2D *newSpline2D(int n)
{
  int i;
  Spline2D *new = mymalloc(sizeof(*new));
  new->n = n;
  new->x = dmalloc(n);
  new->s = mymalloc(n * sizeof(Spline1D *));
  for(i=0; i<n; ++i) {
    new->x[i] = 0.0;
    new->s[i] = NULL;
  }
  return new;
}

void freeSpline2D(Spline2D *s)
{
  int i;
  for(i=0; i<s->n; ++i)
    freeSpline1D(s->s[i]);
  myfree(s->x);
  myfree(s->s);
  myfree(s);
}

void spline2(Spline2D *s, double yp1, double ypn)
{
  int i;
  for(i=0; i<(s->n); ++i)spline(s->s[i], yp1, ypn);
}

double splint2(Spline2D *s, double x1, double x2, double *dx1, double *dx2)
{
  int k, klo=0, khi=s->n-1;
  double h, a, b, ylo, yhi, dylo, dyhi, ry;
  while(khi-klo > 1) {
    k=(khi+klo)/2;
    if(s->x[k] > x1)khi=k;
    else klo=k;
  }
  h=(s->x[khi])-(s->x[klo]);
  a=((s->x[khi])-x1)/h;
  b=(x1-(s->x[klo]))/h;
  ylo=splint(s->s[klo], x2, &dylo);
  yhi=splint(s->s[khi], x2, &dyhi);
  ry = a*ylo + b*yhi;
  *dx1 = (yhi-ylo)/h;
  *dx2 = a*dylo+b*dyhi;
  return ry;
}

