/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: spline7.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include <stdlib.h>
#include "spline.h"
#include "ansiconvert.h"
#include "memory.h"

int WriteSpline1D(FILE *f, Spline1D *s)
{
	if(WriteANSIInt(f, &(s->n), 1) ||
	   WriteANSIDouble(f, s->x, s->n) ||
	   WriteANSIDouble(f, s->y, s->n) ||
	   WriteANSIDouble(f, s->d, s->n)) return 1;
	return 0;
}

Spline1D *ReadSpline1D(FILE *f)
{
	int n;
	Spline1D *s;
	if(ReadANSIInt(f, &n, 1)) return NULL;
	s = newSpline1D(n);
	if(ReadANSIDouble(f, s->x, s->n) ||
	   ReadANSIDouble(f, s->y, s->n) ||
	   ReadANSIDouble(f, s->d, s->n)) {
	  freeSpline1D(s);
	  return NULL;
	}
	return s;
}

int WriteSpline2D(FILE *f, Spline2D *s)
{
	int i;
	if(WriteANSIInt(f, &(s->n), 1) ||
	   WriteANSIDouble(f, s->x, s->n)) return 1;
	for(i=0; i<s->n; ++i) 
	  if(WriteSpline1D(f, s->s[i])) return 1;
	return 0;
}

Spline2D *ReadSpline2D(FILE *f)
{
	int i, n;
	Spline2D *s;
	if(ReadANSIInt(f, &n, 1)) return NULL;
	s = newSpline2D(n);
	if(ReadANSIDouble(f, s->x, s->n)) {
	  freeSpline2D(s);
	  return NULL;
	}
	for(i=0; i<s->n; ++i) {
	  s->s[i] = ReadSpline1D(f);
	  if(s->s[i] == NULL) {
	    freeSpline2D(s);
	    return NULL;
	  }
	}

	return s;
}
